;;; DO NOT MODIFY THIS FILE
(if (featurep 'skk-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "skk/_pkg.el")

(package-provide 'skk :version 1.06 :type 'regular)

;;;***

;;;### (autoloads (skk-isearch-mode-cleanup skk-isearch-mode-setup) "skk-isearch" "skk/skk-isearch.el")

(autoload 'skk-isearch-mode-setup "skk-isearch" "\
hook function called when skk isearch begin." nil nil)

(autoload 'skk-isearch-mode-cleanup "skk-isearch" "\
Hook function called when skk isearch is done." nil nil)

;;;***

;;;### (autoloads (skk-tutorial) "skk-tut" "skk/skk-tut.el")

(autoload 'skk-tutorial "skk-tut" "\
SKK チュートリアルを起動する。" t nil)

;;;***

;;;### (autoloads (vip-mode) "vip" "skk/vip.el")

(autoload 'vip-mode "vip" "\
Turn on VIP emulation of VI." t nil)

;;;***

(provide 'skk-autoloads)
